/*
  gbook.c
  
  GBook - A web page guestbook!
  
  
  By Bill Kendrick
  kendrick@zippy.sonoma.edu
  http://zippy.sonoma.edu/kendrick/
  
  September 22, 1996 - September 24, 1996

  Upgrades:
    December 3, 1996
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cgi-util.h"
#include "dump.h"
#include "htmlerror.h"
#include "readline.h"


/* The mail command to be used by this CGI: */
/* (To find out what it is, you can type "which mail" at your shell prompt) */

#define MAIL_CMD "/bin/mail"


/* General #define's: */

#define YES 1
#define NO 0
#define STRLEN 512


/* Prototypes: */

void dumperror(char * fname, char * text);
void copyrest(FILE * fi, FILE * fo, int keep);
FILE * fopenwitherror(char * filename, char * mode);
void cleanup(char * str);


/* --- MAIN --- */

int main(int argc, char * argv[])
{
  char mailto[STRLEN], subject[STRLEN], name[STRLEN], email[STRLEN],
    url[STRLEN], comment[2048], from[STRLEN], temp[STRLEN];
  int newontop, showemail, showurl, showcomment, showfrom, postit, keep,
    additional, i;
  char addname[100][STRLEN], addval[100][STRLEN];
  FILE * pi, * fi, * fo;
  
  
  /* Initialize CGI: */
  
  cgiinit();
  printf("Content-type: text/html\n\n");
  
  
  /* Get user-defined options: */
  
  getentry(mailto, "_MAILTO");                  /* e-mail addr. to send to  */
  postit = getentryyesno("_POSTIT", YES);       /* post it for public view? */
  newontop = getentryyesno("_NEWONTOP", YES);   /* new ones: top or bottom? */
  showemail = getentryyesno("_SHOWEMAIL", YES); /* show signer's e-mail?    */
  showurl = getentryyesno("_SHOWURL", YES);     /* show signer's url?       */
  showcomment = getentryyesno("_SHOWCOMMENT", YES);        /* show comment? */
  showfrom = getentryyesno("_SHOWFROM", NO);   /* show how signer got here? */
  keep = getentryasint("_KEEP");                /* how many sig's to keep?  */
  
  
  /* Get signer's standard info: */
  
  getentry(name, "_NAME");                    /* their name                 */
  getentry(email, "_EMAIL");                  /* their email address        */
  getentry(url, "_URL");                      /* their website address(URL) */
  getentry(comment, "_COMMENT");              /* any comments they had      */
  getentry(from, "_FROM");                    /* how they got to your page  */
  
  
  /* Get any additional info: */
  
  additional = 0;
  
  for (i = 0; i <= NUM_ENTRIES; i++)
    {
      if (entries[i].name[0] != '_')
	{
	  strcpy(addname[additional], entries[i].name);
	  strcpy(addval[additional], entries[i].val);
	  cleanup(addval[additional]);
	  additional = additional + 1;
	}
    }
  
  
  /* Be sure they filled out enough information! */
  
  if (strlen(name) == 0)
    dumperror("noname.html", "What's your name?");
  
  if (strlen(comment) == 0)
    dumperror("nocomment.html", "What's your comment?");

  
  /* No HTML tags are allowed - they'll mess up the page! */
  
  for (i = 0; i < strlen(comment); i++)
    if (comment[i] == '<' || comment[i] == '>')
      dumperror("notags.html", "HTML tags not allowed!");
  
  cleanup(comment);
  
  
  /* Their e-mail address must be in the right format: */
  
  if (strlen(email) != 0)
    {
      if (strchr(email, '@') == NULL || strchr(email, '.') == NULL ||
	  email[0] == '@' || email[strlen(email) - 1] == '@' ||
	  email[0] == '.' || email[strlen(email) - 1] == '.')
	dumperror("bademail.html",
		  "Incorrect e-mail format! Must be 'username@site'!");
    }
  
  
  /* If they forgot the "http://" in their URL, add it.. or if */
  /* they didn't fill our their URL correctly, quit: */
  
  if (strlen(url) != 0)
    {
      if (strstr(url, ".") == NULL)
	dumperror("badurl.html",
		  "Incorrect URL address! Must be 'http://site.domain/path'!");
      
      if (strstr(url, "http://") == NULL)
	{
	  sprintf(temp, "http://%s", url);
	  strcpy(url, temp);
	}
    }
  
  
  /* E-mail this info, if applicable: */
  
  if (strlen(mailto) != 0)
    {
      sprintf(temp, "%s %s", MAIL_CMD, mailto);
      
      pi = popen(temp, "w");
      if (pi != NULL)
	{
	  /* Mail header: */
	  
	  fprintf(pi, "Subject: GUESTBOOK ADDITION!\n");
	  if (strlen(email) != 0)
	    {
	      fprintf(pi, "From: %s <%s>\n", email, name);
	      fprintf(pi, "Reply-To: %s\n", email);
	    }
	  fprintf(pi, "\n");
	  
	  
	  /* Personal info (BODY): */
	  
	  fprintf(pi, "Name:\t%s\n", name);

	  if (strlen(email) != 0)
	    fprintf(pi, "E-mail:\t%s\n", email);

	  if (strlen(url) != 0)
	    fprintf(pi, "URL:\t%s\n", url);

	  if (strlen(from) != 0)
	    {
	      fprintf(pi, "Found us by:\n");
	      fprintf(pi, "\t%s\n", from);
	    }

	  fprintf(pi, "\n");
	  
	  
	  /* Comments (BODY): */
	  
	  fprintf(pi, "Comments:\n");
	  fprintf(pi, "%s\n", comment);
	  
	  
	  /* Additional (user-defined) info (BODY): */
	  
	  for (i = 0; i < additional; i++)
	    fprintf(pi, "%s\t%s\n", addname[i], addval[i]);
	  
	  
	  /* End the message (BODY): */
	  
	  fprintf(pi, "\n\n");
	  fprintf(pi, "-END\n\n");
	  
	  pclose(pi);
	}
    }
  
  
  /* Post the info. to a page (if applicable): */
  
  if (postit == YES)
    {
      /* Open temp file (will become new guestbook): */
      
      fo = fopenwitherror("temp.dat", "w");
      
      
      /* Copy "top of page" file onto temp file: */
      
      fi = fopenwitherror("top.html", "r");
      copyrest(fi, fo, 0);
      
      
      /* Place marker in temp file: */
      
      fprintf(fo, "\n<! MARKER >\n");
      
      
      /* If new entries don't go on the top, then dump current entries first */
      
      if (newontop == NO)
	{
	  /* Skip all up until the marker... */

	  fi = fopenwitherror("guests.html", "r");
	  do
	    {
	      readline(fi, temp);
	    }
	  while (!feof(fi) && strcmp(temp, "<! MARKER >") != 0);

	  copyrest(fi, fo, keep);
	}
      
      fprintf(fo, "\n<hr>\n");
      fprintf(fo, "\n<ul>\n");
      
      fprintf(fo, "<!-- KEEP -->\n");

      
      /* Dump personal info: */
      
      fprintf(fo, "<li><b>Name:</b> %s\n", name);
      
      if (strlen(email) != 0 && showemail == YES)
	fprintf(fo, "<li><b>E-mail:</b> <a href=\"mailto:%s\">%s</a>\n", email,
		email);
      
      if (strlen(url) != 0 && showurl == YES)
	fprintf(fo, "<li><b>Website:</b> <a href=\"%s\">%s</a>\n", url, url);
      
      if (strlen(from) != 0 && showfrom == YES)
	fprintf(fo, "<li><b>Came to my page from:</b> %s\n", from);
      
      
      /* Dump comments: */
      
      if (showcomment == YES)
	{
	  fprintf(fo, "<li><b>Comments:</b>\n");
	  fprintf(fo, "%s\n", comment);
	}
      
      
      /* Dump additional (user-defined) info: */
	  
      for (i = 0; i < additional; i++)
	fprintf(fo, "<li><b>%s:</b> %s\n", addname[i], addval[i]);
      
      fprintf(fo, "</ul>\n\n");
      

      /* If new entries DO go on the top, then dump current entries last */
      
      if (newontop == YES)
	{
	  fi = fopenwitherror("guests.html", "r");
	  do
	    {
	      readline(fi, temp);
	    }
	  while (!feof(fi) && strcmp(temp, "<! MARKER >") != 0);
	  
	  copyrest(fi, fo, keep);
	}
      
      fclose(fo);
      
      
      /* Copy temp file onto guests.html (thus updating it): */
      
      fi = fopenwitherror("temp.dat", "r");
      fo = fopenwitherror("guests.html", "w");
      
      copyrest(fi, fo, 0);
      
      fclose(fo);
    }
  
  
  /* Send "thank you" message: */
  
  if (dump_no_abort("thanks.html") == -1)
    {
      printf("<h1 align=center>Thanks!</h1>\n");
      printf("Thanks for signing my guestbook!<p>\n");
    }
  
  return(0);
}


/* --- DUMPERROR --- */
/* Dumps an error file - or, if it can't, it displays alternative text. */
/* Exits entire program. */

void dumperror(char * fname, char * text)
{
  if (dump_no_abort(fname) == -1)
    error(text);
  exit(0);
}


/* --- COPYREST --- */
/* Copies the rest of stream fi onto stream fo. */
/* Closes "fi" after it's done. (Does NOT close "fo".) */
/* If "keep" > 0, it keeps that many 'entries' (entries are defined */
/* as sets of lines which contain one line with a "<!-- KEEP -->" in it.) */

void copyrest(FILE * fi, FILE * fo, int keep)
{
  char line[10240];
  int done, many;

  done = 0;
  many = 0;
  
  do
    {
      fgets(line, 10240, fi);
      
      if (!feof(fi))
	{
	  line[strlen(line) - 1] = '\0';

	  if (strcmp(line, "<!-- KEEP -->") == 0)
	    many = many + 1;
	  
	  if (keep != 0 && many > keep)
	    done = 1;
	  
	  fprintf(fo, "%s\n", line);
	}
      else
	done = 1;
    }
  while (!done);
  
  fclose(fi);
}


/* --- FOPENWITHERROR --- */
/* Opens a file stream.  If there's an error, it dumps the "file error" */
/* message and exits. */

FILE * fopenwitherror(char * filename, char * mode)
{
  FILE * fs;
  
  fs = fopen(filename, mode);
  if (fs == NULL)
    dumperror("fileerror.html", "There's been a file error!  Sorry!");
  
  return(fs);
}


/* --- CLEANUP --- */
/* Cleans up a string (converts return carriages into spaces, and then */
/* word-wraps, making it as small (vertically) as possible) */

void cleanup(char * str)
{
  int i, x, lastspace;
  
  /* Convert ^M's and ^J's into spaces: */
  
  for (i = 0; i < strlen(str); i++)
    if (str[i] == 10 || str[i] == 13)
      str[i] = 32;
  
  
  /* Word wrap before the 80th character: */
  
  x = 0;
  lastspace=-1;
  
  for (i = 0; i < strlen(str); i++)
    {
      x = x + 1;
      if (str[i] == 32)
	lastspace = i;
      if (x > 70)
	if (lastspace != -1)
	  {
	    str[lastspace] = '\n';
	    x = x - 70;
	  }
    }
}
